// This program is open-source.
// made by [skaarj] using the example programs for both esp8266 and the level sensor
// place the BSD license here.


#define SSID        "Hole 1"
#define PASS        ""
#define DEST_HOST   "1.1.1.14"  // this is the host I want the sensor to connect to
#define DEST_IP     "1.1.1.14"  // and it's IP address.  SInce I did not define any DNS, put the IP in the host field.
#define TIMEOUT     5000 // mS
#define CONTINUE    false
#define HALT        true

#define ECHO_COMMANDS //SerialUSB output of wifi commands 

#define trigPin 13  // pin trigger radar ultrasonic
#define echoPin 12  // pin answer radar ultrasonic

#define jmp_sel0 6 // ID jumper pins
#define jmp_sel1 5

#define led0  11    // led buton pwr  (pwr0)
#define led1  10    // led buton sel1 (pwr1)
#define led2   9    // led buton sel2 

#define k1     7    // switch k1 
#define k2     8    // selection switch



// display error and reconnect messages

void errorHalt(String msg)
{
  SerialUSB.println(msg);
  SerialUSB.println("HALT");
  ResetConnection();
}


// Read data from WiFi modules and send them to serialusb port until I find whatever I need - or if timeout
boolean echoFind(String keyword)
{
  byte current_char   = 0;
  byte keyword_length = keyword.length();
  
/* v1.1
atentie 
- de tratat cele 3 butoane si cele 3 leduri de pe ele;
- fallback de pe gaura 1 pe gaura 2 pt open acces fara parola
- fallback pe ruter cu useru si parola lu dragos

v1.0 
pus la punct protocolu WiFi
Construit protocol transmisie retea
Calculat parametri radar ultrasonic, gasit eficienta 98% in raza +100mm.....+600mm

*/

#define SSID        "Gaura1"
#define PASS        ""
#define DEST_HOST   "1.1.1.14"
#define DEST_IP     "1.1.1.14"
#define TIMEOUT     5000 // mS
#define CONTINUE    false
#define HALT        true

#define ECHO_COMMANDS //baga pe serialUSB si comenzile trimise la placa WiFi

#define trigPin 13  // pin declansare radar ultrasonic
#define echoPin 12  // pin raspuns radar ultrasonic

#define jmp_sel0 6
#define jmp_sel1 5

#define led0  11    // led buton pwr  (pwr0 in varianta curenta)
#define led1  10    // led buton sel1 (pwr1 in varianta curenta)
#define led2   9    // led buton sel2 

#define k1     7    // buton k1 (pwr 1 in varianta curenta, deci anulat)
#define k2     8    // buton selectie



// afisare mesaje de eroare si reconectare
void errorHalt(String msg)
{
  SerialUSB.println(msg);
  SerialUSB.println("HALT");
  ResetConnection();
}

// citesc date din modulu WiFi si le zbor pe portu serialUSB pana cand gasesc cuvantu cautat sau timeout
boolean echoFind(String keyword)
{
  byte current_char   = 0;
  byte keyword_length = keyword.length();
  
  // daca nu s-a trimis nimic pana in timeout, anulez treaba
  long deadline = millis() + TIMEOUT;  // deadline ii timpu curent + interval timeout
  while(millis() < deadline) // cat timp nu a trecut timeoutu
  {
    if (Serial1.available()) {  // ii disponibil modulu Wifi?
                              char ch = Serial1.read(); // citesc chestii din WiFi
                              SerialUSB.write(ch);  // si le zbor pe serialUSB
                              if (ch == keyword[current_char]) // daca gasesc balarii
                                        if (++current_char == keyword_length) { // si daca ating lungimea cuv
                                                  SerialUSB.println();  // bag linie noua
                                                  return true;  // si confirm ca o reusit treaba
                                        } //if ++curr....
    }//if serial1.available
 }//while
  return false;  // altfel am timeout
}

// Read and echo all available module output.
// (Used when we're indifferent to "OK" vs. "no change" responses or to get around firmware bugs.)


void echoFlush()
  {while(Serial1.available()) SerialUSB.write(Serial1.read());}
  
// Echo module output until 3 newlines encountered.
// (Used when we're indifferent to "OK" vs. "no change" responses.)
void echoSkip()
{
  echoFind("\n");        // Search for nl at end of command echo
  echoFind("\n");        // Search for 2nd nl at end of response.
  echoFind("\n");        // Search for 3rd nl at end of blank line.
}

// Send a command to the module and wait for acknowledgement string
// (or flush module output if no ack specified).
// Echoes all data received to the serial monitor.
boolean echoCommand(String cmd, String ack, boolean halt_on_fail)
{
  Serial1.println(cmd);
  #ifdef ECHO_COMMANDS
    SerialUSB.print("--"); SerialUSB.println(cmd);
  #endif
  
  // If no ack response specified, skip all available module output.
  if (ack == "")
    echoSkip();
  else
    // Otherwise wait for ack.
    if (!echoFind(ack))          // timed out waiting for ack string 
      if (halt_on_fail)
        errorHalt(cmd+" failed");// Critical failure halt.
      else
        return false;            // Let the caller handle it.
  return true;                   // ack blank or ack found
}

// Connect to the specified wireless network.
boolean connectWiFi()
{
  String cmd = "AT+CWJAP=\""; cmd += SSID; cmd += "\",\""; cmd += PASS; cmd += "\"";
  if (echoCommand(cmd, "OK", CONTINUE)) // Join Access Point
  {
    SerialUSB.println("Connected to WiFi.");
    signal(5,127);
    return true;
  }
  else
  {
    SerialUSB.println("Connection to WiFi failed.");
    signal(20,127);
    return false;
  }
}

void ResetConnection() {
                        signal(10,127);
                        echoCommand("AT+RST", "ready", HALT);    // Reset & test if the module is ready  
                        SerialUSB.println("Module is ready.");
                        echoCommand("AT+GMR", "OK", CONTINUE);   // Retrieves the firmware ID (version number) of the module. 
                        echoCommand("AT+CWMODE?","OK", CONTINUE);// Get module access mode. 
  
//                       echoCommand("AT+CWLAP", "OK", CONTINUE); // List available access points - DOESN't WORK FOR ME
  
                         echoCommand("AT+CWMODE=1", "", HALT);    // Station mode
                         echoCommand("AT+CIPMUX=1", "", HALT);    // Allow multiple connections (we'll only use the first).

                        //connect to the wifi
                         boolean connection_established = false;
                         for(int i=0;i<1024;i++)  {
                                                if(connectWiFi()) {
                                                                    connection_established = true;
                                                                    i=1024;
                                                                    break;
                                                }
                         }
                          if (!connection_established) errorHalt("Connection failed");
                          delay(5000);

                          //echoCommand("AT+CWSAP=?", "OK", CONTINUE); // Test connection
                          echoCommand("AT+CIFSR", "", HALT);         // Echo IP address. (Firmware bug - should return "OK".)
                           //echoCommand("AT+CIPMUX=0", "", HALT);      // Set single connection mode
}

void init_input_output_pins() {
                              pinMode (led0, OUTPUT);
                              pinMode (led1, OUTPUT);;
                              pinMode (led2, OUTPUT);
                              pinMode (k1, INPUT);
                              pinMode (k2, INPUT);
                              pinMode (jmp_sel0, INPUT);
                              pinMode (jmp_sel1, INPUT);
}

byte getID() {
 byte ID=0;
 if (digitalRead(jmp_sel0) == HIGH) ID=1;
 if (digitalRead(jmp_sel1) == HIGH) ID=2;
 if (( digitalRead(jmp_sel0) == HIGH) && (digitalRead(jmp_sel1) == HIGH)) ID=3;
 if (( digitalRead(jmp_sel0) == LOW) && (digitalRead(jmp_sel1) == LOW)) ID=4;

return ID;
 
}
void signal(byte frecv, byte pwm) {
  //analogWrite(led2, pwm);
  //PWMC_ConfigureClocks(frecv * 255 , 0, VARIANT_MCK);
}

// ******** SETUP ********
void setup()  
{
  SerialUSB.begin(115200);         // Communication with PC monitor via USB
  Serial1.begin(9600);        // Communication with ESP8266 via 5V/3.3V level shifter
  
  Serial1.setTimeout(TIMEOUT);
  SerialUSB.println("ESP8266 Demo");
  init_input_output_pins();
/*  analogWrite(led2, 127);
  PWMC_ConfigureClocks(10 * 255 , 0, VARIANT_MCK);
*/
  signal(10,127);
  delay(2000);
  ResetConnection();
  digitalWrite(led0, LOW);
  digitalWrite(led1, LOW);
  digitalWrite(led2, LOW);
  
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
}

int getlevel() {
                
                long duration, distance;   //durata, distanta
                digitalWrite(trigPin, LOW);  // reset pin declansare
                delayMicroseconds(2);  // 2 microsecunde pauza
                digitalWrite(trigPin, HIGH);  // activare pin declansare
                delayMicroseconds(10);  // 10 microsecunde pauza
                digitalWrite(trigPin, LOW);  // dezactivare declansare
                duration = pulseIn(echoPin, HIGH);  // citire durata
                //distance = pulseIn(echoPin, HIGH);  // citire distanta
                distance = (duration/2)/29.1;  // calculare distanta
                if (distance - 21 < 5) SerialUSB.println("\nMotorina gogule!!!\n");  // o sarit nivelu?
                if (distance <= 0) SerialUSB.println("out of range!"); // eroare senzor
                if (distance >= 0) return (distance - 21);  // elimin limita minima senzor
                         else return (-1);
                
}

int suici_wifiOK=0;

// ******** LOOP ********
void loop() 
{
  // Establish TCP connection
 String cmd = "AT+CIPSTART=0,\"TCP\",\""; cmd += DEST_IP; cmd += "\",8080";

 while (true) {
              cmd = "AT+CIPSTART=0,\"TCP\",\""; cmd += DEST_IP; cmd += "\",8080";
              if (!echoCommand(cmd, "OK", CONTINUE)) {   // cumva deconectare?
                                                                    // reconectam
                                                                     signal(10,127); 
                                                                    ResetConnection(); 
                                                                     suici_wifiOK=1;               
                                                                  return;
              }
              delay(2000);  
            // stare conexiune
              if (!echoCommand("AT+CIPSTATUS", "OK", CONTINUE)) {   // cumva deconectare?
                                                                    // reconectam
                                                                    signal(10,127);
                                                                    ResetConnection();   
                                                                    suici_wifiOK=1;             
                                                                  return;
              }
            // Build HTTP request.
           // cmd = "GET / HTTP/1.1 Host: "; cmd += DEST_HOST; cmd += ":8080\r\n\r\n";
             
             digitalWrite(led1,HIGH);
             cmd = "Snz ";
             cmd += String(getID());
             cmd += " ";
             cmd +=String(getlevel());
             digitalWrite(led1,LOW);
             cmd +="\r\n\r\n";
            // Ready the module to receive raw data
            if (!echoCommand("AT+CIPSEND=0,"+String(cmd.length()+2), ">", CONTINUE)) {
                                          echoCommand("AT+CIPCLOSE", "", CONTINUE);
                                          SerialUSB.println("Connection timeout.");
                                          signal(10,127);
                                          suici_wifiOK=1;
                                          return;
             }
              // Send the raw HTTP request
              echoCommand(cmd, "OK", CONTINUE);  // GET  
              if (suici_wifiOK == 1) signal(2,40);
              suici_wifiOK=0;
              
              
              
}

  // Loop forever echoing data received from destination server.
  while(true) {
      SerialUSB.write("Not sending data to wifi...\n");   
      echoCommand(cmd, "OK", CONTINUE);
      SerialUSB.write(Serial1.read());
        
    }
      
  errorHalt("ONCE ONLY");
}

/* v1.1
atentie 
- de tratat cele 3 butoane si cele 3 leduri de pe ele;
- fallback de pe gaura 1 pe gaura 2 pt open acces fara parola
- fallback pe ruter cu useru si parola lu dragos

v1.0 
pus la punct protocolu WiFi
Construit protocol transmisie retea
Calculat parametri radar ultrasonic, gasit eficienta 98% in raza +100mm.....+600mm

*/

#define SSID        "Gaura1"
#define PASS        ""
#define DEST_HOST   "1.1.1.14"
#define DEST_IP     "1.1.1.14"
#define TIMEOUT     5000 // mS
#define CONTINUE    false
#define HALT        true

#define ECHO_COMMANDS //baga pe serialUSB si comenzile trimise la placa WiFi

#define trigPin 13  // pin declansare radar ultrasonic
#define echoPin 12  // pin raspuns radar ultrasonic

#define jmp_sel0 6
#define jmp_sel1 5

#define led0  11    // led buton pwr  (pwr0 in varianta curenta)
#define led1  10    // led buton sel1 (pwr1 in varianta curenta)
#define led2   9    // led buton sel2 

#define k1     7    // buton k1 (pwr 1 in varianta curenta, deci anulat)
#define k2     8    // buton selectie



// afisare mesaje de eroare si reconectare
void errorHalt(String msg)
{
  SerialUSB.println(msg);
  SerialUSB.println("HALT");
  ResetConnection();
}

// citesc date din modulu WiFi si le zbor pe portu serialUSB pana cand gasesc cuvantu cautat sau timeout
boolean echoFind(String keyword)
{
  byte current_char   = 0;
  byte keyword_length = keyword.length();
  
  // daca nu s-a trimis nimic pana in timeout, anulez treaba
  long deadline = millis() + TIMEOUT;  // deadline ii timpu curent + interval timeout
  while(millis() < deadline) // cat timp nu a trecut timeoutu
  {
    if (Serial1.available()) {  // ii disponibil modulu Wifi?
                              char ch = Serial1.read(); // citesc chestii din WiFi
                              SerialUSB.write(ch);  // si le zbor pe serialUSB
                              if (ch == keyword[current_char]) // daca gasesc balarii
                                        if (++current_char == keyword_length) { // si daca ating lungimea cuv
                                                  SerialUSB.println();  // bag linie noua
                                                  return true;  // si confirm ca o reusit treaba
                                        } //if ++curr....
    }//if serial1.available
 }//while
  return false;  // altfel am timeout
}

// Read and echo all available module output.
// (Used when we're indifferent to "OK" vs. "no change" responses or to get around firmware bugs.)


void echoFlush()
  {while(Serial1.available()) SerialUSB.write(Serial1.read());}
  
// Echo module output until 3 newlines encountered.
// (Used when we're indifferent to "OK" vs. "no change" responses.)
void echoSkip()
{
  echoFind("\n");        // Search for nl at end of command echo
  echoFind("\n");        // Search for 2nd nl at end of response.
  echoFind("\n");        // Search for 3rd nl at end of blank line.
}

// Send a command to the module and wait for acknowledgement string
// (or flush module output if no ack specified).
// Echoes all data received to the serial monitor.
boolean echoCommand(String cmd, String ack, boolean halt_on_fail)
{
  Serial1.println(cmd);
  #ifdef ECHO_COMMANDS
    SerialUSB.print("--"); SerialUSB.println(cmd);
  #endif
  
  // If no ack response specified, skip all available module output.
  if (ack == "")
    echoSkip();
  else
    // Otherwise wait for ack.
    if (!echoFind(ack))          // timed out waiting for ack string 
      if (halt_on_fail)
        errorHalt(cmd+" failed");// Critical failure halt.
      else
        return false;            // Let the caller handle it.
  return true;                   // ack blank or ack found
}

// Connect to the specified wireless network.
boolean connectWiFi()
{
  String cmd = "AT+CWJAP=\""; cmd += SSID; cmd += "\",\""; cmd += PASS; cmd += "\"";
  if (echoCommand(cmd, "OK", CONTINUE)) // Join Access Point
  {
    SerialUSB.println("Connected to WiFi.");
    signal(5,127);
    return true;
  }
  else
  {
    SerialUSB.println("Connection to WiFi failed.");
    signal(20,127);
    return false;
  }
}

void ResetConnection() {
                        signal(10,127);
                        echoCommand("AT+RST", "ready", HALT);    // Reset & test if the module is ready  
                        SerialUSB.println("Module is ready.");
                        echoCommand("AT+GMR", "OK", CONTINUE);   // Retrieves the firmware ID (version number) of the module. 
                        echoCommand("AT+CWMODE?","OK", CONTINUE);// Get module access mode. 
  
//                       echoCommand("AT+CWLAP", "OK", CONTINUE); // List available access points - DOESN't WORK FOR ME
  
                         echoCommand("AT+CWMODE=1", "", HALT);    // Station mode
                         echoCommand("AT+CIPMUX=1", "", HALT);    // Allow multiple connections (we'll only use the first).

                        //connect to the wifi
                         boolean connection_established = false;
                         for(int i=0;i<1024;i++)  {
                                                if(connectWiFi()) {
                                                                    connection_established = true;
                                                                    i=1024;
                                                                    break;
                                                }
                         }
                          if (!connection_established) errorHalt("Connection failed");
                          delay(5000);

                          //echoCommand("AT+CWSAP=?", "OK", CONTINUE); // Test connection
                          echoCommand("AT+CIFSR", "", HALT);         // Echo IP address. (Firmware bug - should return "OK".)
                           //echoCommand("AT+CIPMUX=0", "", HALT);      // Set single connection mode
}

void init_input_output_pins() {
                              pinMode (led0, OUTPUT);
                              pinMode (led1, OUTPUT);;
                              pinMode (led2, OUTPUT);
                              pinMode (k1, INPUT);
                              pinMode (k2, INPUT);
                              pinMode (jmp_sel0, INPUT);
                              pinMode (jmp_sel1, INPUT);
}

byte getID() {
 byte ID=0;
 if (digitalRead(jmp_sel0) == HIGH) ID=1;
 if (digitalRead(jmp_sel1) == HIGH) ID=2;
 if (( digitalRead(jmp_sel0) == HIGH) && (digitalRead(jmp_sel1) == HIGH)) ID=3;
 if (( digitalRead(jmp_sel0) == LOW) && (digitalRead(jmp_sel1) == LOW)) ID=4;

return ID;
 
}
void signal(byte frecv, byte pwm) {
  //analogWrite(led2, pwm);
  //PWMC_ConfigureClocks(frecv * 255 , 0, VARIANT_MCK);
}

// ******** SETUP ********
void setup()  
{
  SerialUSB.begin(115200);         // Communication with PC monitor via USB
  Serial1.begin(9600);        // Communication with ESP8266 via 5V/3.3V level shifter
  
  Serial1.setTimeout(TIMEOUT);
  SerialUSB.println("ESP8266 Demo");
  init_input_output_pins();
/*  analogWrite(led2, 127);
  PWMC_ConfigureClocks(10 * 255 , 0, VARIANT_MCK);
*/
  signal(10,127);
  delay(2000);
  ResetConnection();
  digitalWrite(led0, LOW);
  digitalWrite(led1, LOW);
  digitalWrite(led2, LOW);
  
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
}

int getlevel() {
                
                long duration, distance;   //durata, distanta
                digitalWrite(trigPin, LOW);  // reset pin declansare
                delayMicroseconds(2);  // 2 microsecunde pauza
                digitalWrite(trigPin, HIGH);  // activare pin declansare
                delayMicroseconds(10);  // 10 microsecunde pauza
                digitalWrite(trigPin, LOW);  // dezactivare declansare
                duration = pulseIn(echoPin, HIGH);  // citire durata
                //distance = pulseIn(echoPin, HIGH);  // citire distanta
                distance = (duration/2)/29.1;  // calculare distanta
                if (distance - 21 < 5) SerialUSB.println("\nMotorina gogule!!!\n");  // o sarit nivelu?
                if (distance <= 0) SerialUSB.println("out of range!"); // eroare senzor
                if (distance >= 0) return (distance - 21);  // elimin limita minima senzor
                         else return (-1);
                
}

int suici_wifiOK=0;

// ******** LOOP ********
void loop() 
{
  // Establish TCP connection
 String cmd = "AT+CIPSTART=0,\"TCP\",\""; cmd += DEST_IP; cmd += "\",8080";

 while (true) {
              cmd = "AT+CIPSTART=0,\"TCP\",\""; cmd += DEST_IP; cmd += "\",8080";
              if (!echoCommand(cmd, "OK", CONTINUE)) {   // cumva deconectare?
                                                                    // reconectam
                                                                     signal(10,127); 
                                                                    ResetConnection(); 
                                                                     suici_wifiOK=1;               
                                                                  return;
              }
              delay(2000);  
            // stare conexiune
              if (!echoCommand("AT+CIPSTATUS", "OK", CONTINUE)) {   // cumva deconectare?
                                                                    // reconectam
                                                                    signal(10,127);
                                                                    ResetConnection();   
                                                                    suici_wifiOK=1;             
                                                                  return;
              }
            // Build HTTP request.
           // cmd = "GET / HTTP/1.1 Host: "; cmd += DEST_HOST; cmd += ":8080\r\n\r\n";
             
             digitalWrite(led1,HIGH);
             cmd = "Snz ";
             cmd += String(getID());
             cmd += " ";
             cmd +=String(getlevel());
             digitalWrite(led1,LOW);
             cmd +="\r\n\r\n";
            // Ready the module to receive raw data
            if (!echoCommand("AT+CIPSEND=0,"+String(cmd.length()+2), ">", CONTINUE)) {
                                          echoCommand("AT+CIPCLOSE", "", CONTINUE);
                                          SerialUSB.println("Connection timeout.");
                                          signal(10,127);
                                          suici_wifiOK=1;
                                          return;
             }
              // Send the raw HTTP request
              echoCommand(cmd, "OK", CONTINUE);  // GET  
              if (suici_wifiOK == 1) signal(2,40);
              suici_wifiOK=0;
              
              
              
}

  // Loop forever echoing data received from destination server.
  while(true) {
      SerialUSB.write("Not sending data to wifi...\n");   
      echoCommand(cmd, "OK", CONTINUE);
      SerialUSB.write(Serial1.read());
        
    }
      
  errorHalt("ONCE ONLY");
}


  // if nothing sent until timeout reached, cancel everything

  long deadline = millis() + TIMEOUT;  // deadline is the current time + timeout interval
  while(millis() < deadline) // while timeout not reached... 
  {
    if (Serial1.available()) {  // Is the wifi module available? 
                              char ch = Serial1.read(); // then I start to read stuff from WiFi
                              SerialUSB.write(ch);  // and throw them to serialUSB
                              if (ch == keyword[current_char]) // If I find any bad stuff 
                                        if (++current_char == keyword_length) { // and I reach the string length
                                                  SerialUSB.println();  // throwing a new line
                                                  return true;  // and we confirm the success 
                                        } //if ++curr....
    }//if serial1.available
 }//while
  return false;  // ...else I have timeout
}

// Read and echo all available module output.
// (Used when we're indifferent to "OK" vs. "no change" responses or to get around firmware bugs.)


void echoFlush()
  {while(Serial1.available()) SerialUSB.write(Serial1.read());}
  
// Echo module output until 3 newlines encountered.
// (Used when we're indifferent to "OK" vs. "no change" responses.)
void echoSkip()
{
  echoFind("\n");        // Search for nl at end of command echo
  echoFind("\n");        // Search for 2nd nl at end of response.
  echoFind("\n");        // Search for 3rd nl at end of blank line.
}

// Send a command to the module and wait for acknowledgement string
// (or flush module output if no ack specified).
// Echoes all data received to the serial monitor.
boolean echoCommand(String cmd, String ack, boolean halt_on_fail)
{
  Serial1.println(cmd);
  #ifdef ECHO_COMMANDS
    SerialUSB.print("--"); SerialUSB.println(cmd);
  #endif
  
  // If no ack response specified, skip all available module output.
  if (ack == "")
    echoSkip();
  else
    // Otherwise wait for ack.
    if (!echoFind(ack))          // timed out waiting for ack string 
      if (halt_on_fail)
        errorHalt(cmd+" failed");// Critical failure halt.
      else
        return false;            // Let the caller handle it.
  return true;                   // ack blank or ack found
}

// Connect to the specified wireless network.
boolean connectWiFi()
{
  String cmd = "AT+CWJAP=\""; cmd += SSID; cmd += "\",\""; cmd += PASS; cmd += "\"";
  if (echoCommand(cmd, "OK", CONTINUE)) // Join Access Point
  {
    SerialUSB.println("Connected to WiFi.");
    signal(5,127);
    return true;
  }
  else
  {
    SerialUSB.println("Connection to WiFi failed.");
    signal(20,127);
    return false;
  }
}

void ResetConnection() {
                        signal(10,127);
                        echoCommand("AT+RST", "ready", HALT);    // Reset & test if the module is ready  
                        SerialUSB.println("Module is ready.");
                        echoCommand("AT+GMR", "OK", CONTINUE);   // Retrieves the firmware ID (version number) of the module. 
                        echoCommand("AT+CWMODE?","OK", CONTINUE);// Get module access mode. 
  
//                       echoCommand("AT+CWLAP", "OK", CONTINUE); // List available access points - DOESN't WORK FOR ME
  
                         echoCommand("AT+CWMODE=1", "", HALT);    // Station mode
                         echoCommand("AT+CIPMUX=1", "", HALT);    // Allow multiple connections (we'll only use the first).

                        //connect to the wifi
                         boolean connection_established = false;
                         for(int i=0;i<1024;i++)  {
                                                if(connectWiFi()) {
                                                                    connection_established = true;
                                                                    i=1024;
                                                                    break;
                                                }
                         }
                          if (!connection_established) errorHalt("Connection failed");
                          delay(5000);

                          //echoCommand("AT+CWSAP=?", "OK", CONTINUE); // Test connection
                          echoCommand("AT+CIFSR", "", HALT);         // Echo IP address. (Firmware bug - should return "OK".)
                           //echoCommand("AT+CIPMUX=0", "", HALT);      // Set single connection mode
}

void init_input_output_pins() {
                              pinMode (led0, OUTPUT);
                              pinMode (led1, OUTPUT);;
                              pinMode (led2, OUTPUT);
                              pinMode (k1, INPUT);
                              pinMode (k2, INPUT);
                              pinMode (jmp_sel0, INPUT);
                              pinMode (jmp_sel1, INPUT);
}

byte getID() {
 byte ID=0;
 if (digitalRead(jmp_sel0) == HIGH) ID=1;
 if (digitalRead(jmp_sel1) == HIGH) ID=2;
 if (( digitalRead(jmp_sel0) == HIGH) && (digitalRead(jmp_sel1) == HIGH)) ID=3;
 if (( digitalRead(jmp_sel0) == LOW) && (digitalRead(jmp_sel1) == LOW)) ID=4;

return ID;
 
}
void signal(byte frecv, byte pwm) {
  //analogWrite(led2, pwm);
  //PWMC_ConfigureClocks(frecv * 255 , 0, VARIANT_MCK);
}

// ******** SETUP ********
void setup()  
{
  SerialUSB.begin(115200);         // Communication with PC monitor via USB
  Serial1.begin(9600);        // Communication with ESP8266 via 5V/3.3V level shifter
  
  Serial1.setTimeout(TIMEOUT);
  SerialUSB.println("ESP8266 Demo");
  init_input_output_pins();
/*  analogWrite(led2, 127);
  PWMC_ConfigureClocks(10 * 255 , 0, VARIANT_MCK);
*/
  signal(10,127);
  delay(2000);
  ResetConnection();
  digitalWrite(led0, LOW);
  digitalWrite(led1, LOW);
  digitalWrite(led2, LOW);
  
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
}

int getlevel() {
                
                long duration, distance;  
                digitalWrite(trigPin, LOW);  // reset trigger pin
                delayMicroseconds(2);  // 2 microseconds delay
                digitalWrite(trigPin, HIGH);  // engage trigger pin 
                delayMicroseconds(10);  // another 10 microseconds delay
                digitalWrite(trigPin, LOW);  // disengage trigger 
                duration = pulseIn(echoPin, HIGH);  // read time interval
                //distance = pulseIn(echoPin, HIGH);  // read distance
                distance = (duration/2)/29.1;  // now calculate the distance
                if (distance - 21 < 5) SerialUSB.println("\nGas level full, dude!!\n");  // is the level too high?
                if (distance <= 0) SerialUSB.println("out of range!"); // sensor error
                if (distance >= 0) return (distance - 21);  // discard the lower sensor limit
                         else return (-1);
                
}

int suici_wifiOK=0;

// ******** LOOP ********
void loop() 
{
  // Establish TCP connection
 String cmd = "AT+CIPSTART=0,\"TCP\",\""; cmd += DEST_IP; cmd += "\",8080";

 while (true) {
              cmd = "AT+CIPSTART=0,\"TCP\",\""; cmd += DEST_IP; cmd += "\",8080";
              if (!echoCommand(cmd, "OK", CONTINUE)) {   // Did the server disconnected us??
                                                                    // reconnect again
                                                                     signal(10,127); 
                                                                    ResetConnection(); 
                                                                     suici_wifiOK=1;               
                                                                  return;
              }
              delay(2000);  
            // connection status:
              if (!echoCommand("AT+CIPSTATUS", "OK", CONTINUE)) {   // disconnected again?
                                                                    // reconnect
                                                                    signal(10,127);
                                                                    ResetConnection();   
                                                                    suici_wifiOK=1;             
                                                                  return;
              }
            // Build HTTP request.
           // cmd = "GET / HTTP/1.1 Host: "; cmd += DEST_HOST; cmd += ":8080\r\n\r\n";
             
             digitalWrite(led1,HIGH);
             cmd = "Snz ";
             cmd += String(getID());
             cmd += " ";
             cmd +=String(getlevel());
             digitalWrite(led1,LOW);
             cmd +="\r\n\r\n";
            // Ready the module to receive raw data
            if (!echoCommand("AT+CIPSEND=0,"+String(cmd.length()+2), ">", CONTINUE)) {
                                          echoCommand("AT+CIPCLOSE", "", CONTINUE);
                                          SerialUSB.println("Connection timeout.");
                                          signal(10,127);
                                          suici_wifiOK=1;
                                          return;
             }
              // Send the raw HTTP request
              echoCommand(cmd, "OK", CONTINUE);  // GET  
              if (suici_wifiOK == 1) signal(2,40);
              suici_wifiOK=0;
              
              
              
}

  // Loop forever echoing data received from destination server.
  while(true) {
      SerialUSB.write("Not sending data to wifi...\n");   
      echoCommand(cmd, "OK", CONTINUE);
      SerialUSB.write(Serial1.read());
        
    }
      
  errorHalt("ONCE ONLY");
}

